from pandas import DataFrame
from rich.table import Table

from goatpie.helpers import data2table, get_country, get_device, get_domain

# Global setup
# (1) Domains
domains = [
    "example.com",
    "//example.com",
    "\\example.com",
    "http://example.com",
    "https://example.com",
    "https://example.com",
    "https://example.com/stats",
    "https://example.com/?stats=1",
]

# (2) Subdomains
subdomains = [
    "stats.example.com",
    "//stats.example.com",
    "\\stats.example.com",
    "http://stats.example.com",
    "https://stats.example.com",
    "https://stats.example.com",
    "https://stats.example.com/stats",
    "https://stats.example.com/?stats=1",
]


def test_get_domain_with_subdomain() -> None:
    """
    Tests 'get_domain' (keeping subdomain)
    """

    # Assert result
    for url in domains:
        assert get_domain(url, True) == "example.com"

    for url in subdomains:
        assert get_domain(url, True) == "stats.example.com"


def test_get_domain_without_subdomain() -> None:
    """
    Tests 'get_domain' (removing subdomain)
    """

    # Assert result
    for url in domains + subdomains:
        assert get_domain(url, False) == "example.com"


def test_get_country() -> None:
    """
    Tests 'get_country'
    """

    # Assert result
    assert get_country("DE") == "Germany"


def test_get_country_invalid() -> None:
    """
    Tests 'get_country' (invalid 'code')
    """

    # Assert result #1
    assert get_country("") == ""

    # Assert result #2
    assert get_country("XY") == "XY"


def test_get_device() -> None:
    """
    Tests 'get_device'
    """

    # Assert result #1
    for i in range(300, 1000):
        assert get_device(str(i)) == "Phones"

    # Assert result #2
    for i in range(1000, 1100):
        assert get_device(str(i)) == "Large phones, small tablets"

    # Assert result #3
    for i in range(1100, 1920):
        assert get_device(str(i)) == "Tablets and small laptops"

    # Assert result #4
    for i in range(1920, 3000):
        assert get_device(str(i)) == "Computer monitors"

    # Assert result #5
    for i in range(3000, 5000):
        assert get_device(str(i)) == "Computer monitors larger than HD"


def test_get_device_order() -> None:
    """
    Tests 'get_device' (different order, same result)
    """

    # Assert result #1
    assert get_device("480") == "Phones"

    # Assert result #2
    assert get_device("320,480") == "Phones"

    # Assert result #3
    assert get_device("480,320") == "Phones"

    # Assert result #4
    assert get_device("480,320,1") == "Phones"


def test_get_device_invalid() -> None:
    """
    Tests 'get_device' (invalid 'size')
    """

    # Assert result #1
    assert get_device("") == "Unknown devices"

    # Assert result #3
    assert get_device("120") == "Unknown devices"

    # Assert result #2
    assert get_device("pine64") == "Unknown devices"


def test_data2table():
    """
    Tests 'data2table'
    """

    # Setup
    # (1) Data table
    data = DataFrame.from_dict({"c1": ["a", "b"], "c2": ["c", "d"]})

    # Run function
    table = data2table(data, ["blue", "blue"])

    # Assert result
    assert isinstance(table, Table)



import gzip
import io
import logging
import os
import sqlite3
import time
from logging.handlers import RotatingFileHandler
from typing import Optional

import arrow
import pandas
import requests

from .helpers import get_country, get_device, get_domain


class GoatPie:
    """
    'Goatcounter' analytics at your fingertips
    """

    # Base URL
    url: str = "https://example.goatcounter.com"

    # Request headers
    headers: dict = {"Content-type": "application/json"}

    # Request timeout (in seconds)
    timeout: int = 10

    # Database
    db: sqlite3.Connection

    # Minimum update interval (in seconds)
    interval: int = 1 * 60 * 60  # 1 hour

    def __init__(self, url: str, token: str, data_dir: str = ".tmp") -> None:
        """
        Creates 'GoatPie' instance

        :param url: str 'Goatcounter' instance URL
        :param token: str API token
        :param data_dir: str Data directory

        :return: None
        """

        # Store base URL
        self.url = url

        # Add API token
        self.headers["Authorization"] = f"Bearer {token}"

        # Store data directory
        self.data_dir = data_dir

        # Create data directory (if needed)
        os.makedirs(data_dir, exist_ok=True)

        # Initialize logger & set logging level
        self.logger = logging.getLogger(__name__)
        self.logger.setLevel(logging.DEBUG)

        # Create log handler
        handler = RotatingFileHandler(os.path.join(data_dir, "debug.log"), maxBytes=10000)

        # Format logfile content
        formatter = logging.Formatter("%(asctime)s - %(name)s - %(levelname)s - %(message)s")
        handler.setFormatter(formatter)

        # Add handler to logger
        self.logger.addHandler(handler)

        # Connect to database
        db_file = os.path.join(data_dir, "db.sqlite")
        self.db = sqlite3.connect(db_file)

        # Populate database
        self.update()

    def fetch(self, endpoint: str, json: Optional[dict] = None) -> requests.Response:
        """
        Fetches data from 'Goatcounter' API

        :param endpoint: str API endpoint
        :param json: dict JSON body data

        :return: requests.Response
        """

        # If not specified ..
        if json is None:
            # .. assign default value
            json = {}

        # Prepare request
        # (1) Define HTTP method
        method = "post" if endpoint == "export" else "get"

        # (2) Build target URL
        base = f"{self.url}/api/v0/{endpoint}"

        # Call API
        return getattr(requests, method)(base, headers=self.headers, json=json, timeout=self.timeout)

    def export(self, first_id: int = 0) -> int:
        """
        Triggers generation of new export

        :param first_id: int Index of first entry

        :return: int Export identifier
        :throws: Exception Something went wrong
        """

        # Call API
        response = self.fetch("export", {"start_from_hit_id": first_id})

        # Get JSON response data
        data = response.json()

        # If error occurred ..
        if "error" in data:
            # .. report back
            raise Exception(data["error"])

        # Determine remaining API (= rate limit)
        remaining = response.headers["X-Rate-Limit-Remaining"]

        # If rate limit is (about to be) exceeded ..
        if int(remaining) < 2:
            # .. wait until next reset happens
            until_reset = response.headers["X-Rate-Limit-Reset"]

            # .. report back
            self.logger.info("Rate limit exceeded, waiting for reset in %s seconds ..", until_reset)

            # Bide the time
            time.sleep(int(until_reset))

        return data["id"]

    def status(self, idx: int) -> dict:
        """
        Checks progress of triggered export

        :param idx: int Export identifier

        :return: dict Status report
        """

        return self.fetch(f"export/{idx}").json()

    def download(self, idx: int) -> str:
        """
        Initiates download of finished export

        :param idx: int Export identifier

        :return: str Exported CSV data
        :throws: Exception Invalid response code
        """

        # Fetch archive
        response = self.fetch(f"export/{idx}/download")

        # If status code indicates error ..
        if response.status_code not in [200, 202]:
            # .. report back
            raise Exception(f"{response.status_code}: {response.content.decode('utf-8')}")

        # Unzip & stringify data
        return gzip.decompress(response.content).decode("utf-8")

    def update(self, last_update: int = 1 * 60 * 60) -> None:
        """
        Fetches analytics data & inserts it into local database

        :param last_update: int Minimum timespan since last update (in seconds)

        :return: None
        """

        # Define file containing next entry
        id_file = os.path.join(self.data_dir, ".next_idx")

        try:
            # If identifier file not present ..
            if not os.path.exists(id_file):
                # .. create it
                with open(id_file, "w", encoding="utf-8") as file:
                    file.write("0")

            # Get last modification & current time
            modified = arrow.get(os.path.getmtime(id_file))
            just_now = arrow.utcnow().to("local")

            # If time since last modification lesser than this ..
            if arrow.get(modified) > just_now.shift(seconds=-last_update):
                # .. do not update database (unless forced to)
                if last_update > 0:
                    # Get human-readable interval since last export
                    interval = arrow.get(modified).humanize()
                    feedback = interval if interval == "just now" else f"less than {interval}"

                    # Report back
                    raise Exception(f"Last update was {feedback}, skipping ..")

            # Append data if database table already exists
            if_exists = "append"

            # Attempt to ..
            try:
                # .. load identifier of next entry
                with open(id_file, "r", encoding="utf-8") as file:
                    first_id = int(file.read())

            # .. but if something goes south & table already exists ..
            except Exception:
                # .. replace it instead of appending to it
                if_exists = "replace"

            # Initiate data export & get export index
            idx = self.export(first_id or 0)

            # Enter indefinite loop
            while True:
                # Wait a ..
                time.sleep(1)

                # Receive export status
                status = self.status(idx)

                # Check export status
                if status["finished_at"] is None:
                    continue

                # If no new data available ..
                if status["num_rows"] == 0:
                    # .. store current time (preventing further requests)
                    timestamp = arrow.utcnow().to("local").timestamp()
                    os.utime(id_file, (timestamp, timestamp))

                    # .. report back
                    raise Exception("No new data, skipping ..")

                # Fetch string containing CSV data
                content = self.download(idx)

                # Load data into dataframe & store in database
                df = pandas.read_csv(io.StringIO(content))
                df.to_sql("data", self.db, if_exists=if_exists, index=False)

                # Update identifier of last entry
                with open(id_file, "w", encoding="utf-8") as file:
                    file.write(str(int(status["last_hit_id"]) + 1))

                # Step out of the loop
                break

        except Exception:
            # Capture stack trace
            self.logger.exception("Something went wrong")

    def execute(self, command: str, params: Optional[dict] = None) -> pandas.DataFrame:
        """
        Executes SQL command against database

        :param command: str SQL command
        :param params: str Query parameters

        :return: pandas.DataFrame
        :throws: Exception Something went wrong
        """

        try:
            return pandas.read_sql(command, self.db, params=params)

        except Exception:
            # Capture stack trace
            self.logger.exception("Something went wrong")

            # Reraise exception
            raise

    def get_referrers(self, limit: int = 12) -> pandas.DataFrame:
        """
        Provides (limited set of) most commonly used referrers

        :param limit: int Limits data to last XY entries

        :return: pandas.DataFrame
        """

        # Load dataframe from database
        # (1) No bots
        # (2) Group by referrer
        # (3) Sort by count
        referrers = self.execute(
            """
            SELECT count(Referrer) as Total, Referrer
            FROM data
            WHERE Bot == 0
            GROUP BY Referrer
            ORDER BY Total DESC
            """
        )

        # Filter out internal links
        # (1) Convert entries to strings (prevent 'None' causing problems)
        referrers["Referrer"] = referrers["Referrer"].astype(str)

        # (2) Remove entries containing domain being used
        referrers = referrers[~referrers["Referrer"].str.contains(get_domain(self.url))]

        # Add percentages
        referrers["%"] = round(referrers["Total"] / referrers["Total"].sum() * 100, 1)

        return referrers.head(limit)

    def get_pages(self, limit: int = 12) -> pandas.DataFrame:
        """
        Provides (limited set of) most commonly visited pages

        :param limit: int Limits data to last XY entries

        :return: pandas.DataFrame
        """

        # Load dataframe from database
        # (1) No bots
        # (2) Group by path
        # (3) Sort by count
        # (4) Last XY entries
        pages = self.execute(
            """
            SELECT count("2Path") as Total, "2Path" as Page
            FROM data
            WHERE Bot == 0
            GROUP BY Page
            ORDER BY Total DESC
            """
        )

        # Add percentages
        pages["%"] = round(pages["Total"] / pages["Total"].sum() * 100, 1)

        return pages.head(limit)

    def get_browsers(self, limit: int = 12) -> pandas.DataFrame:
        """
        Provides (limited set of) most commonly used browsers

        :param limit: int Limits data to last XY entries

        :return: pandas.DataFrame
        """

        # Load dataframe from database
        # (1) No bots
        # (2) Group by browser
        # (3) Sort by count
        # (4) Last XY entries
        browsers = self.execute(
            """
            SELECT count(Browser) as Total, Browser
            FROM data
            WHERE Bot == 0
            GROUP BY Browser
            ORDER BY Total DESC
            """
        )

        # Add percentages
        browsers["%"] = round(browsers["Total"] / browsers["Total"].sum() * 100, 1)

        return browsers.head(limit)

    def get_systems(self, limit: int = 12) -> pandas.DataFrame:
        """
        Provides (limited set of) most commonly used operating systems

        :param limit: int Limits data to last XY entries

        :return: pandas.DataFrame
        """

        # Load dataframe from database
        # (1) No bots
        # (2) Group by system
        # (3) Sort by count
        systems = self.execute(
            """
            SELECT count(System) as Total, System
            FROM data
            WHERE Bot == 0
            GROUP BY System
            ORDER BY Total DESC
            """
        )

        # Add percentages
        systems["%"] = round(systems["Total"] / systems["Total"].sum() * 100, 1)

        return systems.head(limit)

    def get_pageviews(self, limit: int = 14) -> pandas.DataFrame:
        """
        Provides (limited set of) pageviews (= total hits) per day

        :param limit: int Limits data to last XY days

        :return: pandas.DataFrame
        """

        # Load dataframe from database
        # (1) No bots
        # (2) Group by day
        # (3) Sort by day
        pageviews = self.execute(
            """
            SELECT strftime("%Y-%m-%d", Date) as Day, count(*) as Pageviews
            FROM data
            WHERE Bot == 0
            GROUP BY Day
            ORDER BY Day DESC
            """
        )

        return pageviews.head(limit)

    def get_visits(self, limit: int = 14) -> pandas.DataFrame:
        """
        Provides (limited set of) visits (= unique users) per day

        :param limit: int Limits data to last XY days

        :return: pandas.DataFrame
        """

        # Load dataframe from database
        # (1) No bots
        # (2) Group by day
        # (3) Sort by day
        visits = self.execute(
            """
            SELECT strftime("%Y-%m-%d", Date) as Day, count(distinct Session) as Visits
            FROM data
            WHERE Bot == 0
            GROUP BY Day
            ORDER BY Day DESC
            """
        )

        return visits.head(limit)

    def get_devices(self) -> pandas.DataFrame:
        """
        Provides device types (according to screen sizes)

        :return: pandas.DataFrame
        """

        # Load dataframe from database
        # (1) No bots
        # (2) Group by screen size
        devices = self.execute(
            """
            SELECT count("Screen size") as Total, "Screen size" as Device
            FROM data
            WHERE Bot == 0
            GROUP BY Device
            """
        )

        # Replace screen size with device type
        devices["Device"] = devices["Device"].map(get_device)

        # Group device types & aggregate their totals
        devices = devices.groupby("Device").agg({"Total": sum, "Device": "first"})

        # Add percentages
        devices["%"] = round(devices["Total"] / devices["Total"].sum() * 100, 1)

        # Sort by count
        return devices.sort_values(by="Total", ascending=False)

    def get_countries(self, limit: int = 12) -> pandas.DataFrame:
        """
        Provides (limited set of) most common countries

        :param limit: int Limits data to last XY entries

        :return: pandas.DataFrame
        """

        # Load dataframe from database
        # (1) No bots
        # (2) Group by country
        # (3) Sort by count
        countries = self.execute(
            """
            SELECT count(Location) as Total, Location as Country
            FROM data
            WHERE Bot == 0
            GROUP BY Location
            ORDER BY Total DESC
            """
        )

        # Replace country codes with actual country names
        # (1) Convert entries to strings (prevent 'None' causing problems)
        countries["Country"] = countries["Country"].astype(str)

        # (2) Replace entries
        countries["Country"] = countries["Country"].map(get_country)

        # Add percentages
        countries["%"] = round(countries["Total"] / countries["Total"].sum() * 100, 1)

        return countries.head(limit)



import plotext
from rich.ansi import AnsiDecoder
from rich.console import Console, Group
from rich.jupyter import JupyterMixin


class plotextRenderable(JupyterMixin):
    """
    'Rich' renderable for plots created with 'plotext'
    """

    # Plot contents
    rich_canvas: Group

    def __init__(self, obj, limit: int, step: int, bar_colors: list) -> None:
        """
        Initializes 'plotextRenderable' instance

        :param obj: goatpie.GoatPie
        :param limit: int Limits data being displayed
        :param step: int Steps used on Y-axis
        :param bar_colors: list Bar colors

        :return: None
        """

        # Set plotter & settings
        self.obj = obj
        self.limit = limit
        self.step = step
        self.bar_colors = bar_colors

        # Implement decoder
        self.decoder = AnsiDecoder()

    def __rich_console__(self, console: Console, options: dict) -> None:
        """
        Creates canvas for display in the console

        :param console: Console
        :param options: dict

        :return: None
        """

        # Determine dimensions
        width = options.max_width or console.width
        height = options.height or console.height

        # Create canvas from plotted stats
        canvas = self.render_stats(width, height)
        self.rich_canvas = Group(*self.decoder.decode(canvas))

        yield self.rich_canvas

    def render_stats(self, width: int, height: int) -> str:
        """
        Renders visits & pageviews using 'plotext'

        :param width: int Plot width
        :param height: int Plot height

        :return: str Plotted canvas
        """

        # Clear everything
        plotext.clf()
        plotext.clc()

        # Retrieve data
        visits = self.obj.get_visits(self.limit)
        pageviews = self.obj.get_pageviews(self.limit)

        # Configure plot
        plotext.multiple_bar(
            visits.Day,
            [visits.Visits, pageviews.Pageviews],
            label=["Visits", "Pageviews"],
            width=0.4,
            marker="hd",
            color=self.bar_colors,
        )
        plotext.plotsize(width, height)
        plotext.xlabel("hits / day")
        plotext.frame(False)
        plotext.ticks_color("white")
        plotext.ticks_style("bold")
        plotext.xticks([])
        plotext.yticks(range(0, pageviews.Pageviews.max(), self.step))

        return plotext.build()



import configparser
import os

import click

from .app import GoatPieApp
from .goatpie import GoatPie
from .helpers import get_domain


@click.command()
@click.argument("url")
@click.option("-u", "--update", is_flag=True, help="Forces update of local database")
@click.option("-l", "--limit", default=14, help="Shows visits & pageviews for the last XY days. Default: 14")
@click.option("-b", "--bar-colors", type=str, help="Defines two bar colors. Default: 'blue,magenta'")
@click.option("-t", "--text-colors", type=str, help="Defines three text colors. Default: 'blue,white,cyan'")
@click.version_option("0.4.0")
def cli(url: str, update: bool, limit: int, bar_colors: str, text_colors: str) -> None:
    """
    Provides 'Goatcounter' statistics for URL
    """

    # Determine base directory
    base_dir = click.get_app_dir("goatpie")

    # Configure application
    # (1) Set minimum update interval (in seconds)
    interval = 1 * 60 * 60  # 1 hour

    # (2) Set steps used on Y-axis
    step = 100

    # (3) Set bar colors
    bar_colors_default = "blue,magenta"

    # (4) Set text colors
    text_colors_default = "blue,white,cyan"

    # Initialize config object (using defaults)
    config = configparser.ConfigParser(
        defaults={
            "interval": interval,
            "step": step,
            "bar_colors": bar_colors_default,
            "text_colors": text_colors_default,
        }
    )

    # Get path to config file
    config_file = os.path.join(base_dir, "config.ini")

    # If not existent ..
    if not os.path.exists(config_file):
        # .. create it (using default values)
        with open(config_file, "w", encoding="utf-8") as file:
            config.write(file)

    # Load its contents
    config.read(config_file)

    # Get domain identifier
    domain = get_domain(url, True)

    # If no section for domain ..
    if domain not in config:
        # .. create it & add interval option
        config.add_section(domain)
        config.set(domain, "interval", str(interval))
        config.set(domain, "step", str(step))
        config.set(domain, "bar_colors", bar_colors_default)
        config.set(domain, "text_colors", text_colors_default)

    # If no API token for domain section ..
    if not config.has_option(domain, "token"):
        # .. retrieve it
        token = click.prompt("Please enter your token", hide_input=True, confirmation_prompt=True)

        # If approved ..
        if click.confirm("Save token for later?", default=True):
            # (1) .. save API token
            config.set(domain, "token", token)

            # (2) .. update config file
            with open(config_file, "w", encoding="utf-8") as file:
                config.write(file)

    # Get ready
    obj = GoatPie(url, config[domain].get("token"), base_dir)
    obj.interval = int(config[domain].get("interval", interval))

    # If specified ..
    if update:
        # .. force database update
        obj.update(0)

    # Create application
    app = GoatPieApp

    # Configure it
    app.obj = obj
    app.limit = limit
    app.step = int(config[domain].get("step", step))

    app.bar_colors = (
        config[domain].get("bar_colors", bar_colors_default).split(",")
        if bar_colors is None
        else bar_colors.split(",")
    )

    app.text_colors = (
        config[domain].get("text_colors", text_colors_default).split(",")
        if text_colors is None
        else text_colors.split(",")
    )

    # Run!
    app.run(title='"Goatcounter" analytics', log=os.path.join(base_dir, "app.log"))



import urllib
from typing import Optional

import pandas
import pycountry
from rich import box
from rich.table import Column, Table


def get_domain(url: str, keep_subdomain: bool = False) -> str:
    """
    Extract domain ('example.com'), quick&dirty

    :param url: str Target URL
    :param keep_subdomain: bool Whether to strip subdomain

    :return: str
    """

    # Parse URL
    parsed = urllib.parse.urlparse(url)

    # If input is malformed (eg no scheme, like 'example.com') ..
    if parsed.path == url:
        # .. prepend scheme & try again
        parsed = urllib.parse.urlparse(f"http://{url}")

    # Determine domain
    # (1) Extract network location
    # (2) Remove backslashes (eg '\\192.168.X.Y')
    netloc = parsed.netloc.lstrip("\\")

    # If not keeping subdomain ..
    if not keep_subdomain:
        # .. combine its last two parts (= domain name & tld)
        return ".".join(netloc.split(".")[-2:])

    # .. otherwise keep whole record
    return netloc


def get_device(size: str) -> str:
    """
    Converts screen size to device type (if applicable)

    :param size: str Screen size (w,h,dpi)

    :return: str Device type (if matched), otherwise 'unknown'
    """

    # Remove whitespaces
    size = "".join(size.split())

    try:
        # Determine largest value (width OR height)
        size = int(max(map(int, size.split(","))))

    except ValueError:
        # .. report back
        return "Unknown devices"

    # Map screen size to device type
    if size >= 3000:
        return "Computer monitors larger than HD"

    if size >= 1920:
        return "Computer monitors"

    if size >= 1100:
        return "Tablets and small laptops"

    if size >= 1000:
        return "Large phones, small tablets"

    if size >= 300:
        return "Phones"

    return "Unknown devices"


def get_country(code: str) -> str:
    """
    Converts country code to country name (if applicable)

    :param code: str Country code

    :return: str Country name (if successful), otherwise country code
    """

    # Strip surrounding whitespaces
    code = code.strip()

    # Look up country code
    country = pycountry.countries.get(alpha_2=code)

    return code if country is None else country.name


def data2table(data: pandas.DataFrame, colors: list, title: Optional[str] = None) -> Table:
    """
    Creates 'rich' table from 'pandas' dataframe

    :param data: pandas.DataFrame Table data
    :param colors: list Column text colors
    :param title: str Table heading

    :return: rich.table.Table
    """

    # Create header row
    header = [Column(header=header, justify="right", style=colors[idx]) for idx, header in enumerate(data.columns)]

    # Construct table
    table = Table(
        title=title,
        title_style="bold white",
        header_style="bold white",
        show_lines=False,
        box=box.ROUNDED,
        expand=True,
        *header,
    )

    # Add rows to it
    for row in data.values:
        table.add_row(*[str(cell) for cell in row])

    return table



from rich.padding import Padding
from rich.panel import Panel
from textual.widget import Widget

from .plotext import plotextRenderable


class GoatBar(Widget):
    """
    Textual widget for 'multiple_bar' plot created with 'plotext'
    """

    def __init__(self, obj, limit: int, step: int, bar_colors: list) -> None:
        """
        Initializes 'GoatBar' instance

        :param obj: goatpie.GoatPie
        :param limit: int Bar colors
        :param step: int Limits data being displayed
        :param bar_colors: list Steps used on Y-axis

        :return: None
        """

        # Initialize parent
        super().__init__("GoatBar")

        # Set plotter & settings
        self.obj = obj
        self.limit = limit
        self.step = step
        self.bar_colors = bar_colors

    def render(self) -> Panel:
        """
        Renders 'multiple_bar' plot created with 'plotext'

        :return: rich.panel.Panel
        """

        # Create 'rich' element
        renderable = plotextRenderable(self.obj, self.limit, self.step, self.bar_colors)

        # Define top padding
        spacing = (1, 0, 0, 0)

        return Padding(Panel(renderable, title=f"Last {self.limit} days", padding=spacing), spacing)

    # FIX: Update data & rerender
    # async def update(self, renderable: RenderableType) -> None:
    #     self.renderable = renderable
    #     self.refresh()



from click.testing import CliRunner

from goatpie.cli import cli


def test_cli():
    """
    Tests CLI interface
    """

    # Setup
    # (1) CLI runner
    runner = CliRunner()

    # Run function
    result = runner.invoke(cli)

    # Assert result
    assert result.exit_code == 2



from rich.console import Console, Group
from rich.padding import Padding
from textual.app import App
from textual.widgets import ScrollView

from goatpie import GoatPie
from goatpie.components import GoatBar
from goatpie.helpers import data2table


class GoatPieApp(App):
    """
    'Goatcounter' analytics at your fingertips
    """

    # 'GoatPie' instance
    obj: GoatPie

    # Bar colors
    bar_colors: list

    # Column text colors
    text_colors: list

    # Limits data being displayed
    limit: int

    # Steps used on Y-axis
    step: int

    async def on_load(self) -> None:
        """
        Bind keys

        :return: None
        """

        await self.bind("q", "quit", "Quit application")

        # Since 'arrow up' & 'arrow down' work out-of-the-box,
        # binding keys to quit the application suffices for now

    async def on_mount(self) -> None:
        """
        Load widgets

        :return: None
        """

        # Plot visits & pageviews
        plot = GoatBar(self.obj, self.limit, self.step, self.bar_colors)

        # Retrieve data
        referrers = self.obj.get_referrers()
        pages = self.obj.get_pages()
        browsers = self.obj.get_browsers()
        systems = self.obj.get_systems()
        devices = self.obj.get_devices()
        countries = self.obj.get_countries()

        # Create tables from it & group them together
        tables = Group(
            Padding(data2table(referrers, self.text_colors)),
            Padding(data2table(pages, self.text_colors)),
            Padding(data2table(browsers, self.text_colors)),
            Padding(data2table(systems, self.text_colors)),
            Padding(data2table(devices, self.text_colors)),
            Padding(data2table(countries, self.text_colors)),
        )

        # Query console
        console = Console()

        if console.width < 90:
            # Build scrollable view from plot & data tables
            scroll_view = ScrollView(Group(plot, tables))

            # Focus it (enabling 'arrow up/down')
            await self.set_focus(scroll_view)

            # Assign widgets
            await self.view.dock(scroll_view, edge="top")

        else:
            # Build scrollable view from data tables
            table_view = ScrollView(Padding(tables, (1, 0, 0, 0)))

            # Focus it (enabling 'arrow up/down')
            await self.set_focus(table_view)

            # Assign widgets
            await self.view.dock(plot, edge="left", size=int(console.width * 3 / 5))
            await self.view.dock(table_view, edge="right")
